/*

Game Hub Module

Games may use this module to create a live link from the hub for the purpose
of collecting contestants or spectators.

Many options may be overriden or dissabled by overriding the initial value.
Use "" to dissable a value, rather than null, because null will be replaced
by the default value.

*/

#include "hubqry.dm"

GameHub
	proc
		OpenLink()
		CloseLink()
		ReportScores(scores[]) //player keys and their associated scores

		//these may be overridden to change the default behavior
		InviteToHost(mob/player=usr)
		ConfirmScores(player[],score[])
		ReportError(msg)
		HandleLinkUp()
		HandleLinkDown()

	var
		path                 //example: OnlineGames.Tournament.PipeStock
		mob/manager          //the player who is in charge
		has_private_hub_area //true if game has private live area
		web_path             //path to web page for this game in the hub
		last_path            //the last piece in the path
		live_path            //currently, this is live.[last_path]
		priv_link            //link from private directory ([path].live)
		live_link            //link from public directory  ([live_path])
		static/HubQuery/ghub //the hub communicator

GameHub/proc/Text2List(str,delim)
	var
		start = 1
		end = findtext(str,delim)
		lst[0]

	while(end)
		lst += copytext(str,start,end)
		start = end+1
		end = findtext(str,delim,start)
	lst += copytext(str,start)
	return lst

GameHub/New(game_path)
	if(!ghub) ghub = new()
	if(game_path) path = game_path
	if(path==null)
		path = "Live.[cKey(world.name)]"

	var/plist[] = Text2List(path,".")
	if(plist.len) last_path = plist[plist.len]

	if(live_path==null && plist.len)
		live_path = "Live.[plist[plist.len]]"

	if(has_private_hub_area == null && !findtext(path,"live.",1,6))
		//make links from both the private area and the global Live directory
		has_private_hub_area = 1

	if(web_path==null)
		var/p

		if(plist.len && cmptext(plist[1],"live"))
			web_path = "http://www.byond.com/hub/hubcgi.dmb/live"
		else
			//NOTE: for now, do not try to direct web to private area, because it may not always exist
			//if(!has_private_hub_area) plist.Remove(last_path)
			plist.Remove(last_path)

			for(p in plist)
				if(web_path) web_path += "/"
				web_path += ckey(p)

			web_path = "http://www.byond.com/hub/[web_path]/index.html"
			//if(!has_private_hub_area) web_path = "[web_path]#[ckey(last_path)]"
			web_path = "[web_path]#[ckey(last_path)]"

GameHub/OpenLink()
	if(priv_link) ghub.CloseLink(priv_link)
	if(live_link) ghub.CloseLink(live_link)

	//Open up the public and private links
	if(live_path)
		live_link = ghub.OpenLink(live_path,manager.key)
	if(has_private_hub_area && (live_link || !live_path))
		priv_link = ghub.OpenLink("[path].Live",manager.key)

	if(live_link || priv_link) HandleLinkUp()

	return priv_link || live_link

GameHub/CloseLink()
	var/changed = priv_link || live_link
	if(priv_link)
		ghub.CloseLink(priv_link)
		priv_link = null
	if(live_link)
		ghub.CloseLink(live_link)
		live_link = null

	if(changed) HandleLinkDown()

GameHub/HandleLinkUp()
	manager << "<span class='system hub'>A link from the hub has been opened for you.  You may <a href=#\ref[src]close>close</a> it at any time.</span>"

GameHub/HandleLinkDown()
	manager << "<span class='system hub'>The link from the hub has been closed.</span>"

GameHub/InviteToHost(mob/player=usr)
	manager = player
	manager << "<span class=hub>If you wish to invite more players, you may <a href=#\ref[src]open>open</a> a link from the <a href=[web_path]>hub</a> to this game.</span>"

GameHub/Topic(href)
	switch(href)
		if("open") return OpenLink()
		if("close") return CloseLink()
		else return ..()

GameHub/ConfirmScores(scores[])
	var/p
	var/suppressed
	var/note

	world << "<span class=hub>Reporting scores:</span>"
	for(p in scores)
		var/client/C
		for(C)
			if(C.key == p) break

		note = C ? "" : " (not logged in)"
		if(!C) suppressed += 1

		world << "<span class=hub>[p] [scores[p]][note]</span>"

		if(!C) scores.Remove(p)

	if(suppressed)
		world << "<span class='system hub'>Scores of players who are not logged in will not be submitted.</span>"

	//by default, we automatically accept these scores
	return 1

GameHub/ReportScores(scores[])
	var/qparams[0]
	var/p
	for(p in scores)
		var/s = scores[p]
		if(ismob(p) || istype(p,/client)) qparams[p:key] = s
		else if(istext(p)) qparams[p] = s
		else ReportError("Invalid score entry: [p]")

	if(!ConfirmScores(qparams)) return
	if(!length(qparams)) return
	if(!has_private_hub_area)
		ReportError("The path '[path]' cannot contain permanent rank information.")
		return

	qparams[QUERY_PRM] = ADD_SCORES_QRY
	qparams[PATH_PRM] = "[path].Ranks"

	return ghub.SendQuery(qparams)

GameHub/ReportError(msg)
	if(manager && manager.client.address)
		world << msg
	world.log << msg
